import pytest
import pandas as pd

from exchange_calendars.exchange_calendar_xosl import XOSLExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestXOSLCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XOSLExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        # The XOSL is open from 9:00 am to 4:30 pm.
        yield 7.5

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            # 2018
            "2018-01-01",  # New Year's Day
            "2018-03-29",  # Maundy Thursday
            "2018-03-30",  # Good Friday
            "2018-04-02",  # Easter Monday
            "2018-05-01",  # Labour Day
            "2018-05-10",  # Ascension Day
            "2018-05-17",  # Constitution Day
            "2018-05-21",  # Whit Monday
            "2018-12-24",  # Christmas Eve
            "2018-12-25",  # Christmas Day
            "2018-12-26",  # Boxing Day
            "2018-12-31",  # New Year's Eve
        ]

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            # Holidays that fall on a weekend and are not made up. Ensure surrounding
            # days are not holidays.
            # In 2010, Labour Day fell on a Saturday, so the market should be
            # open on both the prior Friday and the following Monday.
            "2010-04-30",
            "2010-05-03",
            # In 2015, Constitution Day fell on a Sunday, so the market should
            # be open on both the prior Friday and the following Monday.
            "2015-05-15",
            "2015-05-18",
            # In 2010, Christmas fell on a Saturday, meaning Boxing Day fell on
            # a Sunday. The market should thus be open on the following Monday.
            "2010-12-27",
            # In 2017, New Year's Day fell on a Sunday, so the market should be
            # open on both the prior Friday and the following Monday.
            "2016-12-30",
            "2017-01-02",
        ]

    @pytest.fixture
    def early_closes_sample(self):
        # 2011 was first year that Holy Wednesday observed an early close.
        yield ["2011-04-20"]

    @pytest.fixture
    def early_closes_sample_time(self):
        yield pd.Timedelta(13, "h")

    @pytest.fixture
    def non_early_closes_sample(self):
        # 2010 was last year before Holy Wednesday started to observe an early close.
        yield ["2010-03-31"]

    @pytest.fixture
    def non_early_closes_sample_time(self):
        yield pd.Timedelta(hours=16, minutes=20)



from datetime import date

import pandas as pd
import pytest

from exchange_calendars.exchange_calendar_xkrx import XKRXExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase
from .test_utils import T


class TestXKRXCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XKRXExchangeCalendar

    @pytest.fixture
    def start_bound(self):
        yield T("1956-01-01")

    @pytest.fixture
    def end_bound(self):
        yield T("2050-12-31")

    @pytest.fixture
    def max_session_hours(self):
        # Korea exchange is open from 9am to 3:30pm
        yield 6.5

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            # 2017
            "2017-01-27",
            "2017-01-30",
            "2017-03-01",
            "2017-05-01",
            "2017-05-03",
            "2017-05-05",
            "2017-05-09",
            "2017-06-06",
            "2017-08-15",
            "2017-10-02",
            "2017-10-03",
            "2017-10-04",
            "2017-10-05",
            "2017-10-06",
            "2017-10-09",
            "2017-12-25",
            "2017-12-29",
            "2010-06-06",  # Memorial Day on Sunday
            #
            # Chuseok holidays falling on a weekend and subsequently made up.
            "2014-09-10",  # falls on Sunday, made up following Wednesday
            "2015-09-29",  # falls on Saturday, made up following Tuesday
            "2017-10-06",
            "2013-10-09",  # Hangeul_day, observance commenced this year
            #
            # Revised alternate holiday rule
            # Since 2021-08-04, the alternative holiday rule, which previously
            # applied to Children's Day only, now also applies to the followings:
            #  - Independence Movement Day (03-01)
            #  - National Liberation Day (08-15)
            #  - Korean National Foundation Day (10-03)
            #  - Hangul Proclamation Day (10-09)
            # National Liberation Day on Sunday
            # so the next monday becomes alternative holiday
            "2021-08-16",
            # Korean National Foundation Day on Sunday
            # so the next monday becomes alternative holiday
            "2021-10-04",
            # Hangul Proclamation Day on Saturday
            # so the next monday becomes alternative holiday
            "2021-10-11",
            # korean thanks giving day on sunday
            # so the next monday becomes alternative holiday
            "2022-09-12",
            # Hangul Proclamation Day on Saturday
            # so the next monday becomes alternative holiday
            "2022-10-10",
            # Buddha's birthday was on 27th May (Saturday),
            # so the next monday becomes alternative holiday
            "2023-05-29",
        ]

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            # Holidays that fall on a weekend and are not made up. Ensure surrounding
            # days are not holidays.
            # National Foundation Day on a Saturday, so check
            # Friday and Monday surrounding it
            "2015-10-02",
            "2015-10-05",
            # Christmas Day on a Saturday
            # Same as Foundation Day idea
            "2010-12-24",
            "2010-12-27",
            "2012-10-09",  # Hangeul_day, last year before observance commenced.
        ]

    @pytest.fixture
    def adhoc_holidays_sample(self):
        yield [
            # Temporary Public Holiday (Added to create a 6-day long holiday period)
            "2025-01-27",
        ]

    # TODO: Issue #94
    def test_late_opens(self, default_calendar, late_opens):  # noqa: ARG002
        # overrides base to mark as xfail
        msg = "Calendar has late opens although `late_opens` is empty. Issue #94"
        pytest.xfail(msg)

    # Calendar-specific tests

    def test_historical_regular_holidays_fall_into_precomputed_holidays(
        self,
        default_calendar,
    ):
        cal = default_calendar
        precomputed = pd.DatetimeIndex(cal.adhoc_holidays)
        # precomputed holidays should not include weekends (saturday, sunday)
        assert (precomputed.weekday < 5).all()

        regular = cal.regular_holidays.holidays(
            precomputed.min(),
            pd.Timestamp("2021-08-15"),
            return_name=True,
        )

        # filter non weekend generated holidays
        non_weekend_regular = regular[regular.index.weekday < 5]

        # regular holidays should generally fall into one of the precomputed holidays
        # except the future holidays that are not precomputed yet
        bv = non_weekend_regular.index.isin(precomputed)
        assert bv.all(), f"missing holidays = \n{non_weekend_regular[~bv]}"

    def test_feb_29_2022_in_lunar_calendar(self, default_calendar):
        # This test asserts that the following does not throw an exception.
        default_calendar.regular_holidays.holidays(date(2022, 3, 31), date(2022, 3, 31))



import pytest

from exchange_calendars.exchange_calendar_xlim import XLIMExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestXLIMCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XLIMExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        # The XLIM is open from 9:00AM to 4:00PM.
        yield 7

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            "2025-06-07",  # Battle of Arica and Flag Day
            "2025-07-23",  # Peruvian Air Force Day
            "2025-08-06",  # Battle of Junín
            "2025-12-09",  # Battle of Ayacucho
            "2019-01-01",  # New Year's Day
            "2019-04-18",  # Maundy Thursday
            "2019-04-19",  # Good Friday
            "2019-05-01",  # Labour Day
            "2018-06-29",  # St. Peter and St. Paul Day
            "2016-07-28",  # Independence Day 1
            "2016-07-29",  # Independence Day 2
            "2019-08-30",  # Santa Rosa
            "2019-10-08",  # Battle of Angamos
            "2019-11-01",  # All Saints' Day
            "2017-12-08",  # Immaculate Conception
            "2019-12-25",  # Christmas Day
            #
            # New Year's Eve ceased being observed after 2007.
            "2005-12-31",
            "2006-12-31",
            "2007-12-31",
        ]

    @pytest.fixture
    def adhoc_holidays_sample(self):
        yield [
            # Exchange holidays.
            "2009-01-02",
            "2009-07-27",
            "2015-01-02",
            "2015-07-27",
            "2015-10-09",
            # ASPA Summit.
            "2012-10-01",
            "2012-10-02",
            # APEC Summit.
            "2016-11-17",
            "2016-11-18",
            # 8th Summit of the Americas.
            "2018-04-13",
        ]

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            # Holidays that fall on a weekend are not made up. Ensure surrounding
            # days are not holidays.
            # New Year's Day on a Sunday.
            "2016-12-30",
            "2017-01-02",
            # Labour Day (May 1st) on a Sunday.
            "2016-04-29",
            "2016-05-02",
            # Saint Peter and Saint Paul Day (June 29th) on a Saturday.
            "2019-06-28",
            "2019-07-01",
            # Independence Days (July 28th and 29th) on a Saturday and Sunday.
            "2018-07-27",
            "2018-07-30",
            # Santa Rosa (August 30th) on a Sunday.
            "2015-08-28",
            "2015-08-31",
            # Battle of Angamos (October 8th) on a Sunday.
            "2017-10-06",
            "2017-10-09",
            # All Saints' Day (November 1st) on a Sunday.
            "2015-10-30",
            "2015-11-02",
            # Immaculate Conception (December 8th) on a Sunday.
            "2019-12-06",
            "2019-12-09",
            # Christmas on a Sunday.
            "2016-12-23",
            "2016-12-26",
            #
            # New Year's Eve ceased being observed after 2007.
            "2008-12-31",
            "2009-12-31",
            "2010-12-31",
        ]



from zoneinfo import ZoneInfo

import pandas as pd
import pytest

from exchange_calendars.exchange_calendar_asex import ASEXExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestASEXCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield ASEXExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        # The ASEX is open from 10:00 to 5:20PM on its longest trading day
        yield 7 + (1 / 3)

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            # 2019
            "2019-01-01",  # New Year's Day
            "2017-01-06",  # Epiphany
            "2019-03-11",  # Orthodox Ash Monday
            "2019-03-25",  # National Holiday
            "2019-04-19",  # Good Friday
            "2019-04-22",  # Easter Monday
            "2019-04-26",  # Orthodox Good Friday
            "2019-04-29",  # Orthodox Easter Monday
            "2019-05-01",  # Labour Day
            "2019-06-17",  # Orthodox Whit Monday
            "2019-08-15",  # Assumption Day
            "2019-10-28",  # National Holiday
            "2019-12-24",  # Christmas Eve
            "2019-12-25",  # Christmas Day
            "2019-12-26",  # Second Day of Christmas
            #
            # The Athens Stock Exchange observes Orthodox (or Eastern) Easter,
            # as well as Western Easter.  All holidays that are tethered to
            # Easter (i.e. Whit Monday, Good Friday, etc.), are relative to
            # Orthodox Easter. Following tests that Orthodox Easter and all
            # related holidays are correct.
            # Some Orthodox Easter dates
            "2005-05-01",
            "2006-04-23",
            "2009-04-19",
            "2013-05-05",
            "2015-04-12",
            "2018-04-08",
            # Some Orthodox Good Friday dates
            "2002-05-03",
            "2005-04-29",
            "2008-04-25",
            "2009-04-17",
            "2016-04-29",
            "2017-04-14",
            # Some Orthodox Whit Monday dates
            "2002-06-24",
            "2005-06-20",
            "2006-06-12",
            "2008-06-16",
            "2013-06-24",
            "2016-06-20",
            # Some Orthodox Ash Monday dates
            "2002-03-18",
            "2005-03-14",
            "2007-02-19",
            "2011-03-07",
            "2014-03-03",
            "2018-02-19",
        ]

    @pytest.fixture
    def adhoc_holidays_sample(self):
        adhocs = [
            "2002-05-07",  # Market Holiday
            "2004-08-13",  # Assumption Day makeup
            "2008-03-04",  # Worker strikes
            "2008-03-05",  # Worker strikes
            "2013-05-07",  # May Day strikes
            "2014-12-31",  # New Year's Eve
            "2016-05-03",  # Labour Day makeup
        ]
        # 2015 Greek debt crisis closed markets for an extended period.
        # Following ensures there are no sessions over this time.
        crisis_dates = pd.date_range("2015-06-29", "2015-07-31")
        yield adhocs + crisis_dates.strftime("%Y-%m-%d").to_list()

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            # Holidays NOT made up despite falling on weekend. Following ensures
            # surrounding days are not holidays.
            # New Years Day on Sunday, Jan 1st
            "2011-12-30",
            "2012-01-02",
            # Epiphany on Sunday, Jan 6th
            "2019-01-04",
            "2019-01-07",
            # National Holiday on Sunday, Mar 25th
            "2018-03-23",
            "2018-03-26",
            # Labour Day on Sunday, May 1st
            "2011-04-29",
            "2011-05-02",
            # Assumption Day on Saturday, Aug 15th
            "2015-08-14",
            "2015-08-17",
            # National Holiday on Saturday, Oct 28
            "2015-10-27",
            "2015-10-30",
            # Christmas Eve on a Sunday. NB 25th and 26th both holidays.
            "2017-12-22",
            "2017-12-27",
            # Christmas on a Sunday. NB 26th a holiday.
            "2016-12-23",
            "2016-12-27",
            # 2nd Day of Christmas on Saturday, Dec 26th. NB 25th, 24th both holidays.
            "2015-12-23",
            "2015-12-28",
        ]

    # Calendar-specific tests

    def test_close_time_change(self, default_calendar):
        """
        On Sept 29, 2008, the ASEX decided to push its close time back
        from 5:00PM to 5:20PM to close the time gap with Wall Street.
        """
        cal = default_calendar
        close_time = cal.closes["2006-09-29"]
        tz = ZoneInfo("Europe/Athens")
        assert close_time == pd.Timestamp("2006-09-29 17:00", tz=tz)

        close_time = cal.closes["2008-09-26"]
        assert close_time == pd.Timestamp("2008-09-26 17:00", tz=tz)

        close_time = cal.closes["2008-09-29"]
        assert close_time == pd.Timestamp("2008-09-29 17:20", tz=tz)

        close_time = cal.closes["2008-09-30"]
        assert close_time == pd.Timestamp("2008-09-30 17:20", tz=tz)



import pytest
from exchange_calendars.exchange_calendar_xtal import XTALExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestXTALCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XTALExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        # The XTAL is open from 10:00 am to 4:00 pm.
        yield 6

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            # 2025 sample year
            "2025-01-01",  # New Year's Day
            "2025-02-24",  # Independence Day
            "2025-04-18",  # Good Friday
            "2025-04-21",  # Easter Monday
            "2025-05-01",  # Labour Day
            "2025-06-23",  # Victory Day
            "2025-06-24",  # Midsummer Day
            "2025-08-20",  # Restoration of Independence
            "2025-12-24",  # Christmas Eve
            "2025-12-26",  # Boxing Day
            "2025-12-31",  # New Year's Eve
        ]

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            # Days around holidays that should be open
            "2025-01-02",
            "2025-02-25",
            "2025-04-17",
            "2025-05-02",
            "2025-08-21",
            "2025-12-23",
            "2025-12-25",
            "2025-12-30",
        ]



from datetime import time
from zoneinfo import ZoneInfo

import pandas as pd
from pandas.tseries.holiday import EasterMonday, GoodFriday, Holiday

from .common_holidays import (
    all_saints_day,
    ascension_day,
    assumption_day,
    boxing_day,
    christmas_eve,
    european_labour_day,
    new_years_day,
    new_years_eve,
)
from .exchange_calendar import HolidayCalendar, ExchangeCalendar

NewYearsDay = new_years_day()
LabourDay = european_labour_day()
AscensionDay = ascension_day()
AssumptionDay = assumption_day()
AllSaintsDay = all_saints_day()
ChristmasEve = christmas_eve()
BoxingDay = boxing_day()
NewYearsEve = new_years_eve()

RestorationOfTheState = Holiday("Restoration of the State", month=2, day=16)
RestorationOfIndependence = Holiday("Restoration of Independence", month=3, day=11)
StJohnsDay = Holiday("St. John's Day", month=6, day=24)
StatehoodDay = Holiday("Statehood Day", month=7, day=6)
AllSoulsDay = Holiday("All Souls' Day", month=11, day=2)


class XLITExchangeCalendar(ExchangeCalendar):
    """
    Calendar for the Vilnius Stock Exchange in Lithuania.
    https://nasdaqbaltic.com/statistics/en/calendar?holidays=1

    Open Time: 10:00 AM, EET (Eastern European Time)
    Close Time: 4:00 PM, EET (Eastern European Time)

    Regularly-Observed Holidays:
      - New Year's Day
      - Restoration of the State (Feb 16)
      - Restoration of Independence (Mar 11)
      - Good Friday
      - Easter Monday
      - Labour Day (May 1)
      - Ascension Day
      - St. John's Day (Jun 24)
      - Statehood Day (Jul 6)
      - Assumption Day (Aug 15)
      - All Saints' Day (Nov 1)
      - All Souls' Day (Nov 2)
      - Christmas Eve (Dec 24)
      - Boxing Day (Dec 26)
      - New Year's Eve (Dec 31)
    """

    name = "XLIT"
    tz = ZoneInfo("Europe/Vilnius")
    open_times = ((None, time(10, 0)),)
    close_times = ((None, time(16, 0)),)

    @property
    def regular_holidays(self):
        return HolidayCalendar(
            [
                NewYearsDay,
                RestorationOfTheState,
                RestorationOfIndependence,
                GoodFriday,
                EasterMonday,
                LabourDay,
                AscensionDay,
                StJohnsDay,
                StatehoodDay,
                AssumptionDay,
                AllSaintsDay,
                AllSoulsDay,
                ChristmasEve,
                BoxingDay,
                NewYearsEve,
            ]
        )

    @property
    def adhoc_holidays(self):
        return [
            pd.Timestamp("2023-12-25"),  # Additional Day off
        ]



from datetime import time
from zoneinfo import ZoneInfo

import pandas as pd
from pandas.tseries.holiday import (
    Holiday,
    EasterMonday,
    GoodFriday,
)

from .common_holidays import (
    new_years_day,
    epiphany,
    european_labour_day,
    corpus_christi,
    assumption_day,
    all_saints_day,
    christmas_eve,
    christmas,
    new_years_eve,
)
from .exchange_calendar import HolidayCalendar, ExchangeCalendar

NewYearsDay = new_years_day()
Epiphany = epiphany()
LabourDay = european_labour_day()
NationalDayFrom2020 = Holiday("National Day", month=5, day=30, start_date="2020")
NationalDayFrom2002To2020 = Holiday(
    "National Day", month=6, day=25, start_date="2002", end_date="2020"
)
NationalDayFrom1996To2002 = Holiday(
    "National Day", month=5, day=30, start_date="1996", end_date="2002"
)
CorpusChristi = corpus_christi(start_date="2002")
AntiFascistStruggleDay = Holiday("Anti-Fascist Struggle Day", month=6, day=22)
VictoryDay = Holiday("Victory and Homeland Thanksgiving Day", month=8, day=5)
AssumptionDay = assumption_day()
IndependenceDay = Holiday(
    "Independence Day", month=10, day=8, start_date="2002", end_date="2020"
)
AllSaintsDay = all_saints_day()
RemembranceDay = Holiday("Remembrance Day", month=11, day=18, start_date="2020")
ChristmasEve = christmas_eve()
Christmas = christmas()
StStephensDay = Holiday("St. Stephen's Day", month=12, day=26)
NewYearsEve = new_years_eve()


class XZAGExchangeCalendar(ExchangeCalendar):
    """
    Calendar for the Zagreb Stock Exchange (Croatia).
    https://zse.hr/en/non-trading-days/110

    Open Time: 9:00 AM
    Close Time: 4:00 PM

    Regularly-Observed Holidays:
      - New Year's Day (Jan 1)
      - Epiphany (Jan 6)
      - Good Friday
      - Easter Monday
      - Labour Day (May 1)
      - National Day (May 30)
      - Corpus Christi (movable, 60 days after Easter)
      - Anti-Fascist Struggle Day (Jun 22)
      - Victory and Homeland Thanksgiving Day (Aug 5)
      - Assumption Day (Aug 15)
      - All Saints' Day (Nov 1)
      - Remembrance Day (Nov 18)
      - Christmas Eve (Dec 24)
      - Christmas Day (Dec 25)
      - St. Stephen's Day (Dec 26)
      - New Year's Eve (Dec 31)
    """

    name = "XZAG"
    tz = ZoneInfo("Europe/Zagreb")
    open_times = ((None, time(9, 0)),)
    close_times = ((None, time(16, 0)),)

    @property
    def regular_holidays(self):
        return HolidayCalendar(
            [
                NewYearsDay,
                Epiphany,
                GoodFriday,
                EasterMonday,
                LabourDay,
                NationalDayFrom2020,
                NationalDayFrom2002To2020,
                NationalDayFrom1996To2002,
                CorpusChristi,
                AntiFascistStruggleDay,
                VictoryDay,
                AssumptionDay,
                IndependenceDay,
                AllSaintsDay,
                RemembranceDay,
                ChristmasEve,
                Christmas,
                StStephensDay,
                NewYearsEve,
            ]
        )

    @property
    def adhoc_holidays(self):
        return [
            pd.Timestamp("2024-04-17"),  # Election Day
            pd.Timestamp("2022-12-30"),  # Conversion of HRK to EUR
            pd.Timestamp("2022-12-29"),  # Conversion of HRK to EUR
        ]



import pandas as pd
import pytest

from exchange_calendars.exchange_calendar_xbda import XBDAExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestXBDACalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XBDAExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        # The XBDA is open from 9:00 am to 4:30 pm.
        yield 7.5

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            # 2025
            "2025-01-01",  # New Year's Day
            "2025-04-18",  # Good Friday
            "2025-05-23",  # Bermuda Day (the Friday before the last Monday in May)
            "2025-06-16",  # National Heroes Day (third Monday in June)
            "2025-07-31",  # Emancipation Day (Thursday before first Monday in August)
            "2025-08-01",  # Mary Prince Day (Friday after Emancipation Day)
            "2025-09-01",  # Labour Day (first Monday in September)
            "2025-11-11",  # Remembrance Day
            "2025-12-25",  # Christmas Day
            "2025-12-26",  # Boxing Day
            # 2024
            "2024-01-01",  # New Year's Day
            "2024-03-24",  # Good Friday
            "2024-05-24",  # Bermuda Day (the Friday before the last Monday in May)
            "2024-06-17",  # National Heroes Day (third Monday in June)
            "2024-08-01",  # Emancipation Day (Thursday before first Monday in August)
            "2024-08-02",  # Mary Prince Day (Friday after Emancipation Day)
            "2024-09-02",  # Labour Day (first Monday in September)
            "2024-11-11",  # Remembrance Day
            "2024-12-25",  # Christmas Day
            "2024-12-26",  # Boxing Day
            # 2023
            "2023-01-02",  # New Year's Day (observed)
            "2023-04-07",  # Good Friday
            "2023-05-26",  # Bermuda Day (the Friday before the last Monday in May)
            "2023-06-19",  # National Heroes Day (third Monday in June)
            "2023-08-03",  # Emancipation Day (Thursday before first Monday in August)
            "2023-08-04",  # Mary Prince Day (Friday after Emancipation Day)
            "2023-09-04",  # Labour Day (first Monday in September)
            "2023-11-13",  # Remembrance Day (observed, since Nov 11 is a Saturday)
            "2023-12-25",  # Christmas Day
            "2023-12-26",  # Boxing Day
            # 2022
            "2022-01-03",  # New Year's Day (observed)
            "2022-04-15",  # Good Friday
            "2022-05-27",  # Bermuda Day (the Friday before the last Monday in May)
            "2022-06-20",  # National Heroes Day (third Monday in June)
            "2022-07-28",  # Emancipation Day (Thursday before first Monday in August)
            "2022-07-29",  # Mary Prince Day (Friday after Emancipation Day)
            "2022-09-05",  # Labour Day (first Monday in September)
            "2022-11-11",  # Remembrance Day
            "2022-12-26",  # Christmas Day (observed, since Dec 25 is a Sunday)
            "2022-12-27",  # Boxing Day (observed, since Dec 26 is a Monday)
            # 2021
            "2021-01-01",  # New Year's Day
            "2021-04-02",  # Good Friday
            "2021-05-28",  # Bermuda Day (the Friday before the last Monday in May)
            "2021-06-21",  # National Heroes Day (third Monday in June)
            "2021-07-29",  # Emancipation Day (Thursday before first Monday in August)
            "2021-07-30",  # Mary Prince Day (Friday after Emancipation Day)
            "2021-09-06",  # Labour Day (first Monday in September)
            "2021-11-11",  # Remembrance Day
            "2021-12-27",  # Christmas Day (observed, since Dec 25 is a Saturday)
            "2021-12-28",  # Boxing Day (observed, since Dec 26 is a Sunday)
            # 2020
            "2020-01-01",  # New Year's Day
            "2020-04-10",  # Good Friday
            "2020-05-29",  # Bermuda Day (the Friday before the last Monday in May)
            "2020-06-15",  # National Heroes Day (third Monday in June)
            "2020-07-30",  # Emancipation Day (Thursday before first Monday in August)
            "2020-07-31",  # Mary Prince Day (Friday after Emancipation Day)
            "2020-09-07",  # Labour Day (first Monday in September)
            "2020-11-11",  # Remembrance Day
            "2020-12-25",  # Christmas Day
            "2020-12-28",  # Boxing Day (observed, since Dec 26 is a Saturday)
        ]

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            # Days around holidays
            "2025-05-29",  # Thursday before Bermuda Day
            "2025-06-17",  # Tuesday after National Heroes Day
            "2025-08-04",  # Monday after Mary Prince Day
            "2025-12-23",  # Tuesday before Christmas Eve
        ]

    @pytest.fixture
    def early_closes_sample(self):
        yield [
            # Christmas Eve (early close at 2:00 PM)
            "2025-12-24",
            "2024-12-24",
            "2023-12-22",  # In 2023, December 24th falls on a Sunday, so the early close for Christmas Eve is observed on Friday, December 22nd.  # noqa: E501
        ]

    @pytest.fixture
    def early_closes_sample_time(self):
        yield pd.Timedelta(14, "h")



from datetime import time, datetime
from zoneinfo import ZoneInfo

import pandas as pd
from pandas.tseries.holiday import (
    MO,
    TH,
    FR,
    SA,
    DateOffset,
    Holiday,
    GoodFriday,
    next_monday,
    next_monday_or_tuesday,
    previous_friday,
    weekend_to_monday,
)
from .common_holidays import (
    new_years_day,
    christmas_eve,
    christmas,
    boxing_day,
)
from .exchange_calendar import ExchangeCalendar, HolidayCalendar


def bermuda_day_observance(dt: datetime) -> datetime | None:
    """
    How Bermuda Day is observed.
    """
    if dt.year <= 2017 or dt.year == 2019:
        # For 2017 and before, Bermuda Day was on May 24th.
        dt = next_monday(datetime(dt.year, 5, 24))
    elif dt.year <= 2020:
        # For 2020 and before, it was the last Friday in May.
        dt = dt + DateOffset(weekday=FR(-1))
    else:
        # For 2021 and after, it is the Friday before the last Monday in May.
        dt = dt + DateOffset(weekday=MO(-1)) + DateOffset(weekday=FR(-1))
    return dt


def national_heroes_day_observance(dt: datetime) -> datetime | None:
    """
    How National Heroes Day is observed.
    """
    if dt.year == 2008:
        dt = datetime(2008, 10, 13)
    elif dt.year >= 2009:
        dt = dt + DateOffset(weekday=MO(3))
    else:
        dt = None
    return dt


def queens_birthday_observance(dt: datetime) -> datetime | None:
    """
    How Queens Birthday is observed.
    """
    if dt.year <= 1999:
        # For 1999 and before, it was the 3rd Monday in June.
        dt = dt + DateOffset(weekday=MO(3))
    elif dt.year <= 2008:
        # For 2000 to 2008, it is the Monday after the second Saturday in June.
        dt = dt + DateOffset(weekday=SA(2)) + DateOffset(days=2)
    else:
        dt = None
    return dt


NewYearsDay = new_years_day(observance=weekend_to_monday)
ChristmasEve = christmas_eve(observance=previous_friday)
Christmas = christmas(observance=next_monday)
BoxingDay = boxing_day(observance=next_monday_or_tuesday)


# Bermuda Day: the Friday before the last Monday in May.
BermudaDay = Holiday(
    "Bermuda Day",
    month=5,
    day=31,
    observance=bermuda_day_observance,
)

# National Heroes Day: third Monday in June. Replaced Queens Birthday holiday in 2018.
NationalHeroesDay = Holiday(
    "National Heroes Day",
    month=6,
    day=1,
    observance=national_heroes_day_observance,
    start_date=pd.Timestamp("2018-01-01"),
)

QueensBirthday = Holiday(
    "Queens Birthday",
    month=6,
    day=1,
    observance=queens_birthday_observance,
    end_date=pd.Timestamp("2008-12-31"),
)

# Emancipation Day: Thursday before the first Monday in August.
EmancipationDay = Holiday(
    "Emancipation Day",
    month=8,
    day=1,
    offset=[DateOffset(weekday=MO(1)), DateOffset(weekday=TH(-1))],
    start_date=pd.Timestamp("2000-01-01"),
)

# Cup Match Day: Thursday before the first Monday in August. Cup Match was
# renamed to Emancipation Day from 2000.
CupMatchDay = Holiday(
    "Cup Match Day",
    month=8,
    day=1,
    offset=[DateOffset(weekday=MO(1)), DateOffset(weekday=TH(-1))],
    end_date=pd.Timestamp("1999-12-31"),
)

# Mary Prince Day: Friday after Emancipation Day.
MaryPrinceDay = Holiday(
    "Mary Prince Day",
    month=8,
    day=1,
    offset=[DateOffset(weekday=MO(1)), DateOffset(weekday=TH(-1)), DateOffset(days=1)],
    start_date=pd.Timestamp("2020-01-01"),
)

# Somers Day: Friday after Emancipation Day. Somers Day was renamed to Mary
# Prince Day from 2020.
SomersDay = Holiday(
    "Somers Day",
    month=8,
    day=1,
    offset=[DateOffset(weekday=MO(1)), DateOffset(weekday=TH(-1)), DateOffset(days=1)],
    end_date=pd.Timestamp("2019-12-31"),
)

LabourDay = Holiday(
    "Labour Day",
    month=9,
    day=1,
    offset=DateOffset(weekday=MO(1)),
)

RemembranceDay = Holiday(
    "Remembrance Day",
    month=11,
    day=11,
    observance=weekend_to_monday,
)


class XBDAExchangeCalendar(ExchangeCalendar):
    """
    Calendar for the Bermuda Stock Exchange (MIC: XBDA).
    https://www.bsx.com/trading-hours-and-holidays

    Open Time: 9:00 AM, Atlantic/Bermuda
    Close Time: 4:30 PM, Atlantic/Bermuda

    Regularly-Observed Holidays:
      - New Year's Day
      - Good Friday
      - Bermuda Day (the Friday before the last Monday in May)
      - National Heroes Day (third Monday in June)
      - Emancipation Day (Thursday before first Monday in August)
      - Mary Prince Day (Friday after Emancipation Day)
      - Labour Day (first Monday in September)
      - Remembrance Day (November 11, or the following Monday if it falls on a weekend)
      - Christmas Day
      - Boxing Day
    """

    name = "XBDA"
    tz = ZoneInfo("Atlantic/Bermuda")
    open_times = ((None, time(9, 0)),)
    close_times = ((None, time(16, 30)),)
    regular_early_close = time(14, 0)

    @property
    def regular_holidays(self):
        return HolidayCalendar(
            [
                NewYearsDay,
                GoodFriday,
                BermudaDay,
                NationalHeroesDay,
                QueensBirthday,
                EmancipationDay,
                CupMatchDay,
                MaryPrinceDay,
                SomersDay,
                LabourDay,
                RemembranceDay,
                Christmas,
                BoxingDay,
            ]
        )

    @property
    def special_closes(self):
        return [
            (
                self.regular_early_close,
                HolidayCalendar(
                    [
                        ChristmasEve,
                    ]
                ),
            ),
        ]

    @property
    def adhoc_holidays(self):
        return [
            # Coronation of King Charles III
            pd.Timestamp("2023-05-08"),
            # Queen Elizabeth II Funeral
            pd.Timestamp("2022-09-19"),
            # Flora Duffy Day
            pd.Timestamp("2021-10-18"),
            # Portuguese Welcome 170th Anniversary
            pd.Timestamp("2019-11-04"),
            # Public Holiday
            pd.Timestamp("2007-01-05"),
        ]



import pytest
import pandas as pd

from exchange_calendars.exchange_calendar_xidx import XIDXExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestXIDXCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XIDXExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        yield 6 + (5 / 6)

    # Calendar-specific tests

    def test_trading_days(self, default_calendar):
        cal = default_calendar
        # Data obtained from https://www.idx.co.id/en-us/news/trading-holiday/
        expected_trading_days = {
            # year: (trading days in year, [trading days in each month])
            2016: (246, [20, 20, 21, 21, 20, 22, 16, 22, 21, 21, 22, 20]),
            2017: (238, [21, 19, 22, 17, 20, 15, 21, 22, 19, 22, 22, 18]),
            # NOTE: The website says only 21 trading days for July 2018.
            2018: (240, [22, 19, 21, 21, 20, 13, 22, 21, 19, 23, 21, 18]),
            2019: (245, [22, 19, 20, 19, 21, 15, 23, 22, 21, 23, 21, 19]),
            2020: (242, [22, 20, 21, 21, 16, 21, 22, 18, 22, 19, 21, 19]),
            2021: (247, [20, 19, 22, 21, 17, 21, 21, 20, 22, 20, 22, 22]),
            2022: (250, [21, 18, 22, 20, 18, 21, 21, 22, 22, 21, 22, 22]),
            2023: (239, [21, 20, 21, 14, 21, 17, 20, 22, 20, 22, 22, 19]),
            2024: (237, [22, 18, 18, 16, 18, 18, 23, 22, 20, 23, 20, 19]),
            2025: (237, [19, 20, 19, 16, 17, 18, 23, 21, 21, 23, 20, 20]),
        }
        for year, (total_days, monthly_days) in expected_trading_days.items():
            assert sum(monthly_days) == total_days  # Sanity check

            sessions = cal.sessions[cal.sessions.year == year]
            assert total_days == len(sessions), year

    @pytest.mark.parametrize(
        "year, holidays",
        [
            (
                2019,
                [
                    "2019-01-01",  # New Year 2019
                    "2019-02-05",  # Chinese New Year 2570 Kongzili
                    "2019-03-07",  # Hindu Saka New Year 1941
                    "2019-04-03",  # Isra Mikraj of Prophet Muhammad SAW
                    "2019-04-17",  # Indonesia General Election 2019
                    "2019-04-19",  # Good Friday
                    "2019-05-01",  # Labor Day
                    "2019-05-30",  # Ascension Day Of Jesus Christ
                    "2019-06-03",  # Commemoration of Eid ul-Fitr 1440 Hijriyah
                    "2019-06-04",  # Commemoration of Eid ul-Fitr 1440 Hijriyah
                    "2019-06-05",  # Eid ul-Fitr 1440 Hijriyah
                    "2019-06-06",  # Eid ul-Fitr 1440 Hijriyah
                    "2019-06-07",  # Commemoration of Eid ul-Fitr 1440 Hijriyah
                    "2019-12-24",  # Commemoration Of Christmas Day
                    "2019-12-25",  # Christmas Day
                    "2019-12-31",  # Trading Holiday
                ],
            ),
            (
                2018,
                [
                    "2018-01-01",  # New Year 2018
                    "2018-02-16",  # Chinese New Year 2569 Kongzili
                    "2018-03-17",  # Hindu Saka New Year 1940.
                    "2018-03-30",  # Good Friday
                    "2018-04-14",  # Isra Mikraj of Prophet Muhammad SAW
                    "2018-05-01",  # Labor Day
                    "2018-05-10",  # Ascension Day Of Jesus Christ
                    "2018-05-29",  # Vesak Day 2562
                    "2018-06-01",  # Pancasila Day
                    "2018-06-11",  # Commemoration of Eid ul-Fitr 1439 Hijriyah
                    "2018-06-12",  # Commemoration of Eid ul-Fitr 1439 Hijriyah
                    "2018-06-13",  # Commemoration of Eid ul-Fitr 1439 Hijriyah
                    "2018-06-14",  # Commemoration of Eid ul-Fitr 1439 Hijriyah
                    "2018-06-15",  # Eid ul-Fitr 1440 Hijriyah
                    "2018-06-18",  # Commemoration of Eid ul-Fitr 1439 Hijriyah
                    "2018-06-19",  # Commemoration of Eid ul-Fitr 1439 Hijriyah
                    "2018-08-17",  # Independence Day of Indonesia
                    "2018-08-22",  # Eid al-Adha 1439 Hijriyah
                    "2018-09-11",  # Islamic New Year 1442 Hijriyah
                    "2018-11-20",  # Birth of Prophet Muhammad
                    "2018-12-24",  # Commemoration Of Christmas Day
                    "2018-12-25",  # Christmas Day
                    "2018-12-31",  # Trading Holiday
                ],
            ),
        ],
    )
    def test_holidays_in_year(self, default_calendar, year, holidays):
        cal = default_calendar
        days = pd.date_range(start=f"{year}-01-01", end=f"{year}-12-31", freq="B")
        days = days.strftime("%Y-%m-%d")

        for holiday in holidays:
            # Sanity check
            assert holiday in days or pd.Timestamp(holiday).weekday() in (5, 6)

        for day in days:
            if day in holidays:
                assert not cal.is_session(day)
            else:
                assert cal.is_session(day)



import pytest
from exchange_calendars.exchange_calendar_xzag import XZAGExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestXZAGCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XZAGExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        # The XZAG is open from 9:00 am to 4:00 pm.
        yield 7.0

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            "2025-01-01",  # New Year's Day
            "2025-01-06",  # Epiphany
            "2025-04-18",  # Good Friday
            "2025-04-21",  # Easter Monday
            "2025-05-01",  # Labour Day
            "2025-05-30",  # Statehood Day
            "2025-06-19",  # Corpus Christi
            "2025-08-05",  # Victory and Homeland Thanksgiving Day
            "2025-08-15",  # Assumption of Mary
            "2025-11-01",  # All Saints' Day
            "2025-12-25",  # Christmas Day
            "2025-12-26",  # St. Stephen's Day
        ]

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            "2025-01-02",
            "2025-01-07",
            "2025-04-17",
            "2025-05-02",
            "2025-06-18",
            "2025-08-06",
            "2025-08-14",
            "2025-11-03",
            "2025-12-30",
        ]



import pytest
import pandas as pd

from exchange_calendars.exchange_calendar_xmad import XMADExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestXMADCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XMADExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        # The XMAD is open from 9:00 am to 5:30 pm.
        yield 8.5

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            # 2014
            "2014-01-01",  # New Year's Day
            "2014-04-18",  # Good Friday
            "2014-04-21",  # Easter Monday
            "2014-05-01",  # Labour Day
            "2014-12-25",  # Christmas Day
            "2014-12-26",  # Boxing Day
            #
            # Holidays falling on a weekend and subsequently made up.
            "2004-08-16",  # Assumption day, falls Sun 15th, made up on Monday.
            #
            # Christmas Eve was observed as a full day, and then became a half
            # day from 2010 until 2020, and then went back to being a full day
            # from 2021, and then became a half day from 2024.
            "2010-12-24",
            "2021-12-24",
            "2023-12-24",
            #
            # Last observance of holidays that subsequently ceased to be observed.
            "2006-01-06",  # Epiphany
            "2003-08-15",  # Assumption Day
            "2004-10-12",  # National Day
            "2004-11-01",  # All Saints Day
            "2004-12-06",  # Constitution Day
            "2004-12-08",  # Immaculate Conception
            #
            # New Year's Eve was a holiday through to 2010, then became an early
            # close, and then went back to being a full day's holiday from 2021,
            # and then became an early close from 2024.
            "2010-12-31",
            "2021-12-31",
            "2023-12-31",
        ]

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            # Holidays that fall on a weekend and are not made up. Ensure surrounding
            # days are not holidays.
            # In 2010, Labour Day fell on a Saturday, so the market should be
            # open on both the prior Friday and the following Monday.
            "2010-04-30",
            "2010-05-03",
            # Christmas also fell on a Saturday, meaning Boxing Day fell on a
            # Sunday. The market should still be open on the following Monday
            # (note that Christmas Eve was observed as a holiday through 2010,
            # so the market is closed on the previous Friday).
            "2010-12-27",
            #
            # Christmas Eve was an early close from 2012-12-24 to 2020-12-24,
            # and from 2024-12-24
            "2012-12-24",
            "2020-12-24",
            #
            # Regular holidays that ceased to be observed.
            # First subsequent 'would-be' occurrence...
            "2009-01-06",  # Epiphany
            "2005-08-15",  # Assumption Day
            "2005-10-12",  # National Day
            "2005-11-01",  # All Saints Day
            "2005-12-06",  # Constitution Day
            "2005-12-08",  # Immaculate Conception
            # New Year's Eve was holiday through to 2010, but then became an early
            # close after that, before becoming a full market holiday again in 2021,
            # then became an early close in 2024.
            "2012-12-31",
            "2020-12-31",
        ]

    @pytest.fixture
    def early_closes_sample(self):
        yield [
            # Christmas Eve was an early close from 2012 to 2020 and from 2024
            "2012-12-24",
            "2020-12-24",
            "2024-12-24",
            "2025-12-24",
            "2012-12-31",  # New Year's Eve was an early close from 2012 to 2020 and from 2024  # noqa: E501
            "2020-12-31",
            "2024-12-31",
            "2025-12-31",
        ]

    @pytest.fixture
    def early_closes_sample_time(self):
        yield pd.Timedelta(14, "h")



import pytest
from exchange_calendars.exchange_calendar_xbel import XBELExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase


class TestXBELExchangeCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XBELExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        # The XBEL is open from 9:30 am to 2:00 pm.
        yield 4.5

    @pytest.fixture
    def regular_holidays_sample(self):
        yield [
            "2024-01-01",  # New Year's Day
            "2024-01-02",  # New Year's Day
            "2024-01-07",  # Orthodox Christmas
            "2024-02-15",  # Statehood Day
            "2024-02-16",  # Statehood Day Holiday
            "2024-05-01",  # Labour Day
            "2024-05-02",  # Labour Day Holiday
            "2024-05-03",  # Orthodox Good Friday
            "2024-05-06",  # Orthodox Easter Monday
            "2024-11-11",  # Armistice Day
        ]

    @pytest.fixture
    def non_holidays_sample(self):
        yield [
            "2024-01-03",
            "2024-01-15",
            "2024-02-14",
            "2024-03-28",
            "2024-04-02",
            "2024-05-08",
            "2024-05-10",
            "2024-11-12",
            "2024-12-24",
            "2024-12-26",
        ]



import pytest
import pandas as pd

from exchange_calendars.exchange_calendar_xhkg import XHKGExchangeCalendar
from .test_exchange_calendar import ExchangeCalendarTestBase
from .test_utils import T


class TestXHKGCalendar(ExchangeCalendarTestBase):
    @pytest.fixture(scope="class")
    def calendar_cls(self):
        yield XHKGExchangeCalendar

    @pytest.fixture
    def max_session_hours(self):
        yield 6.5

    @pytest.fixture
    def start_bound(self):
        yield T("1960-01-01")

    @pytest.fixture
    def end_bound(self):
        yield T("2049-12-31")

    @pytest.fixture
    def adhoc_holidays_sample(self):
        # Lunar Month 12 2002 is the 12th month of the lunar year that begins
        # in 2002; this month actually takes place in January 2003.

        # Lunar Month 12 2002
        #   January-January
        # Su Mo Tu We Th Fr Sa
        #                 3  4
        #  5  6  7  8  9 10 11
        # 12 13 14 15 16 17 18
        # 19 20 21 22 23 24 25
        # 26 27 28 29 30 31

        #  Lunar Month 1 2003
        #    February-March
        # Su Mo Tu We Th Fr Sa
        #                    1
        #  2  3  4  5  6  7  8
        #  9 10 11 12 13 14 15
        # 16 17 18 19 20 21 22
        # 23 24 25 26 27 28  1
        #  2

        # Prior to 2011, lunar new years eve is a holiday if new years is a Saturday.
        lunar_2003 = ["2003-01-31", "2003-02-03"]

        # Lunar Month 12 2017 is the 12th month of the lunar year that begins
        # in 2017; this month actually takes place in January and February 2018.

        # Lunar Month 12 2017
        #   January-February
        # Su Mo Tu We Th Fr Sa
        #          17 18 19 20
        # 21 22 23 24 25 26 27
        # 28 29 30 31  1  2  3
        #  4  5  6  7  8  9 10
        # 11 12 13 14 15

        #  Lunar Month 1 2018
        #    February-March
        # Su Mo Tu We Th Fr Sa
        #                16 17
        # 18 19 20 21 22 23 24
        # 25 26 27 28  1  2  3
        #  4  5  6  7  8  9 10
        # 11 12 13 14 15 16
        lunar_2018 = ["2018-02-16", "2018-02-19"]

        # 2017 Lunar month 6 will be a leap month (double length). This
        # affects when all the lunisolar holidays after the 6th month occur.

        #                         Gregorian Calendar
        #                                2017
        #
        #        January               February                 March
        # Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa
        #  1  2  3  4  5  6  7             1  2  3  4             1  2  3  4
        #  8  9 10 11 12 13 14    5  6  7  8  9 10 11    5  6  7  8  9 10 11
        # 15 16 17 18 19 20 21   12 13 14 15 16 17 18   12 13 14 15 16 17 18
        # 22 23 24 25 26 27 28   19 20 21 22 23 24 25   19 20 21 22 23 24 25
        # 29 30 31               26 27 28               26 27 28 29 30 31
        #
        #         April                   May                   June
        # Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa
        #                    1       1  2  3  4  5  6                1  2  3
        #  2  3  4  5  6  7  8    7  8  9 10 11 12 13    4  5  6  7  8  9 10
        #  9 10 11 12 13 14 15   14 15 16 17 18 19 20   11 12 13 14 15 16 17
        # 16 17 18 19 20 21 22   21 22 23 24 25 26 27   18 19 20 21 22 23 24
        # 23 24 25 26 27 28 29   28 29 30 31            25 26 27 28 29 30
        # 30
        #         July                  August                September
        # Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa
        #                    1          1  2  3  4  5                   1  2
        #  2  3  4  5  6  7  8    6  7  8  9 10 11 12    3  4  5  6  7  8  9
        #  9 10 11 12 13 14 15   13 14 15 16 17 18 19   10 11 12 13 14 15 16
        # 16 17 18 19 20 21 22   20 21 22 23 24 25 26   17 18 19 20 21 22 23
        # 23 24 25 26 27 28 29   27 28 29 30 31         24 25 26 27 28 29 30
        # 30 31
        #        October               November               December
        # Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa
        #  1  2  3  4  5  6  7             1  2  3  4                   1  2
        #  8  9 10 11 12 13 14    5  6  7  8  9 10 11    3  4  5  6  7  8  9
        # 15 16 17 18 19 20 21   12 13 14 15 16 17 18   10 11 12 13 14 15 16
        # 22 23 24 25 26 27 28   19 20 21 22 23 24 25   17 18 19 20 21 22 23
        # 29 30 31               26 27 28 29 30         24 25 26 27 28 29 30
        #                                               31

        #                           Lunar Calendar
        #                                2017
        #
        #    Lunar Month 1          Lunar Month 2          Lunar Month 3
        #   January-February        February-March          March-April
        # Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa
        #                   28                                28 29 30 31  1
        # 29 30 31  1  2  3  4   26 27 28  1  2  3  4    2  3  4  5  6  7  8
        #  5  6  7  8  9 10 11    5  6  7  8  9 10 11    9 10 11 12 13 14 15
        # 12 13 14 15 16 17 18   12 13 14 15 16 17 18   16 17 18 19 20 21 22
        # 19 20 21 22 23 24 25   19 20 21 22 23 24 25   23 24 25
        #                        26 27
        #
        #    Lunar Month 4          Lunar Month 5         Lunar Month 6(+)
        #      April-May               May-June             June-August
        # Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa
        #          26 27 28 29                  26 27                     24
        # 30  1  2  3  4  5  6   28 29 30 31  1  2  3   25 26 27 28 29 30  1
        #  7  8  9 10 11 12 13    4  5  6  7  8  9 10    2  3  4  5  6  7  8
        # 14 15 16 17 18 19 20   11 12 13 14 15 16 17    9 10 11 12 13 14 15
        # 21 22 23 24 25         18 19 20 21 22 23      16 17 18 19 20 21 22
        #                                               23 24 25 26 27 28 29
        #                                               30 31  1  2  3  4  5
        #                                                6  7  8  9 10 11 12
        #                                               13 14 15 16 17 18 19
        #                                               20 21
        #
        #    Lunar Month 7          Lunar Month 8          Lunar Month 9
        #   August-September      September-October       October-November
        # Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa
        #       22 23 24 25 26            20 21 22 23                  20 21
        # 27 28 29 30 31  1  2   24 25 26 27 28 29 30   22 23 24 25 26 27 28
        #  3  4  5  6  7  8  9    1  2  3  4  5  6  7   29 30 31  1  2  3  4
        # 10 11 12 13 14 15 16    8  9 10 11 12 13 14    5  6  7  8  9 10 11
        # 17 18 19               15 16 17 18 19         12 13 14 15 16 17
        #
        #    Lunar Month 10         Lunar Month 11         Lunar Month 12
        #  November-December       December-January       January-February
        # Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa   Su Mo Tu We Th Fr Sa
        #                   18      18 19 20 21 22 23            17 18 19 20
        # 19 20 21 22 23 24 25   24 25 26 27 28 29 30   21 22 23 24 25 26 27
        # 26 27 28 29 30  1  2   31  1  2  3  4  5  6   28 29 30 31  1  2  3
        #  3  4  5  6  7  8  9    7  8  9 10 11 12 13    4  5  6  7  8  9 10
        # 10 11 12 13 14 15 16   14 15 16               11 12 13 14 15
        # 17
        lunar_2017 = [
            "2017-01-02",  # New Year's Day (Sunday to Monday observance)
            "2017-01-30",  # Lunar New Year
            "2017-01-31",  # Lunar New Year
            "2017-04-04",  # Qingming Festival (off qingming solar term, not lunar)
            "2017-04-14",  # Good Friday
            "2017-04-17",  # Easter Monday
            "2017-05-01",  # Labour Day
            "2017-05-03",  # Buddha's Birthday (The 8th day of the 4th lunar month)
            # Tuen Ng Festival (also known as Dragon Boat Festival. The 5th day
            # of the 5th lunar month, then Sunday to Monday observance)
            "2017-05-30",
            "2017-10-02",  # National Day (Sunday to Monday observance)
            # The day following the Mid-Autumn Festival (Mid-Autumn Festival
            # is the 15th day of the 8th lunar month. This market holiday is
            # next day because the festival is celebrated at night)
            "2017-10-05",
            "2017-12-25",  # Christmas Day
            "2017-12-26",  # The day after Christmas
        ]

        typhoon_days = ["2021-10-13", "2024-09-06"]

        mid_autumn_festival_holidays = [
            "2022-09-12"  # In 2019, the mid-autumn festival holiday was observed on the following business day.  # noqa: E501
        ]

        yield (
            lunar_2003
            + lunar_2018
            + lunar_2017
            + typhoon_days
            + mid_autumn_festival_holidays
        )

    @pytest.fixture
    def early_closes_sample(self):
        yield ["2018-02-15", "2017-01-27"]

    @pytest.fixture
    def early_closes_sample_time(self):
        yield pd.Timedelta(12, "h")
